/*************************
 *
 * Carport-Beleuchtung
 * Finger 2016
 * www.fingers-welt.de
 * Arduino Pro Mini
 *
 *************************/
 
#include <LiquidCrystal.h>
#include "pcf8574.h"
#include <Wire.h>
#include "TimerOne.h" 
		
//---------------
// Variablenzeug
//---------------

// Blinkende LED
volatile uint16_t Blinkzaehler = 0;
#define BlinkFrequenz	50
uint8_t LEDToggle = 0;

// Portabfragen
uint8_t Portwert = 0;

// Timerfunktionen
volatile uint32_t	InternTicker = 0;
volatile uint32_t LichtZaehler = 0;
#define LichtZaehlerMax 1800
volatile uint16_t sTicker = 0;

// Schwellenwerte
#define GrundDauer								120							// 120 Sekunden Grundzeit f�r die Beleuchtung
#define Warnschwelle							 30						  // Sekunde vor Ende Warnung (Gleich ist es dunkel)
#define Blinkdauer								 20							// Zeit für hektisches Blinken
	
volatile uint32_t SollDauerLicht = GrundDauer;		// Grundsätzliche Dauer einer Beleuchtungseinheit
volatile uint8_t  AblaufBlinkzaehler = 0;					// Für hektisches Blinken, wenn ungleich Null

// Steuerung der Lichtintensität
volatile uint8_t Inensitaet = 0;									// Aktuelle Helligkeit
volatile uint8_t LichtAn = 0;											// Flag zur Lichtsteuerung
volatile uint8_t  Dimmerzaehler = 0;							// Für softiges Ein- und Ausdimmen
#define Dimmgeschwindigkeit 		0
#define DimmSchrittweite 				2

// Zeitmessung für Tasterfunktionen
volatile uint16_t TasterZeitMessung  = 0;
volatile uint8_t TasterStatus = 0;

// Melderkontaktfunktionen
volatile uint8_t BewegungsmelderEnable = 1;				// Flag zur Freigabe des Bewegungsmelders
uint8_t MelderKontakt = 0;												// Zur Messung negativer Flanken der Meldekontakte	
uint8_t MelderKontaktAlt = 0;

// Portzuordnungen
#define TasterZwischentuer					1
#define ZwischentuerVerriegelt			2
#define SchiebetuerVerriegelt				4
#define TasterSchiebetuer						8
#define TasterPlatine								16
#define Bewegungsmelder							32
#define ZwischentuerGeoeffnet				64
#define SchiebetuerGeoeffnet				128

// Hardware klarmachen
PCF8574 Port (0x20);  
LiquidCrystal lcd (4, 5, 6, 7, 8, 9);

/***************************
 *
 * System startklar machen
 *
 ***************************/
void setup () 
 {
  pinMode (13, OUTPUT);															// Die LED kann weiter froehlich vor sich hinblinken
  pinMode (10, OUTPUT);															// Relaisausgang für das 12V-Netzteil
	digitalWrite (10, LOW);  													// Das Relais ist aus	

	 // Steuerung der Lichtbaender
	 pinMode (3, OUTPUT);															// Pin 3 steuert die Lichtbänder
	 analogWrite (3, Inensitaet);											// Und er ist aus.....

	// Allgemeines Gerödel
	lcd.begin (8, 2);
	lcd.setCursor(0, 0); lcd.print ("Start");
  delay (1000);
	Timer1.initialize (10000);											  // Timer 1 startklarmachen, Aufruf 10x pro Sekunde	
  Timer1.attachInterrupt (TimerISR);								// ISR einbinden
}

/******************
 *
 * ISR f�r Timer 1
 *
 ******************/
void TimerISR () 
 {
  InternTicker++;																// 100x pro Sekunde raufz�hlen
 
	// Zeitmessung für Tastendruck
	if (TasterZeitMessung > 1) TasterZeitMessung--;	
 
	//--------------------------
	// LED blinkt vor sich hin
	//--------------------------
  Blinkzaehler++;
	if (Blinkzaehler >= BlinkFrequenz)
	 {
	  Blinkzaehler = 0;
		if (LEDToggle) LEDToggle = 0; else LEDToggle = 1;
		if (LEDToggle) digitalWrite (13, HIGH);  else digitalWrite (13, LOW);  
	 }
	
	//---------------------------------------------------------
	// Hektisches Blinken notwendig? Zeit läuft gleich ab....
	//---------------------------------------------------------
	if (AblaufBlinkzaehler > 0)
	 {
	  AblaufBlinkzaehler--;
		if ((Blinkzaehler % 20) == 0) Inensitaet = 255; else Inensitaet = 0;
	 }
	
	//-----------------------------------------
	// Ist Rein- oder Rausdimmen erforderlich?
	//-----------------------------------------
	if ( (LichtAn) && (AblaufBlinkzaehler == 0))
	 { // Es werde heller....
			 if ((Inensitaet+DimmSchrittweite) < 255) Inensitaet += DimmSchrittweite; else Inensitaet = 255;
	 } 
	 
	 if ((!LichtAn) && (AblaufBlinkzaehler == 0))
		 {  // Es werde dunkler...
				 if ((Inensitaet-DimmSchrittweite) > 0) Inensitaet -= DimmSchrittweite; else Inensitaet = 0;
		 } // Von else if (LichtAn)
	 
	//----------------------
	// Licht neu einstellen
	//----------------------
	analogWrite (3, Inensitaet);																													// Und er ist aus..... 
	if ( (Inensitaet == 0) && (AblaufBlinkzaehler == 0) ) digitalWrite (10, LOW);  else 	// Das Relais ist aus	
																												digitalWrite (10, HIGH);  			// Licht ist noch an...										
	 
	//-----------------------------------------------------
	// Steuerung der ablaufenden Zeit, Ablauf jede Sekunde
	//-----------------------------------------------------
	sTicker++;
	if (sTicker > 100)
	 {
			if (LichtZaehler > 0) LichtZaehler--; 
			if (LichtZaehler == 0)
			 {		// Licht aus, Ende hier
							LichtAn = 0;
							BewegungsmelderEnable = 1;																				// Bewegungsmelder wieder freigeben	
							SollDauerLicht = GrundDauer; 																			// Wieder auf Defaultwert setzen
			 } else 
			 { 
			   if (LichtZaehler == Warnschwelle) AblaufBlinkzaehler = Blinkdauer; 		// Hektisches Blinken einleiten
			   LichtAn = 1;
			 } 
		sTicker = 0;
   } // Von if (sTicker > 10)	 
	 
 }

//-----------------------------------------------------------
// Liefert einen Status der Taster (alle drei gleichwertig)
//-----------------------------------------------------------
char GetTaster (void)
 {
    #define KeinTaster					0
		#define DauerTaster					10
		#define KurzTaster					20

		// Abtastzeitraum eine Sekunde
		 uint8_t Tempwert = Port.read8 ();
		 uint8_t TasterZustand = 0;
		 
		// Überführen in handhabbaren Wert
		TasterZustand = 0;
		if ((Tempwert & TasterZwischentuer)==0) 	TasterZustand += 1;
		if ((Tempwert & TasterSchiebetuer)==0) 	TasterZustand += 2;
		if ((Tempwert & TasterPlatine)==0) 			TasterZustand += 4;

    if (TasterStatus != KeinTaster) return (0);
    
		//---------------------------------------------------------------------------------------
		// Neue Zustandserkennung einleiten (=> Zeitmessung starten) wenn Taster gedrückt wurde
		//---------------------------------------------------------------------------------------
		if (TasterZustand != 0) 
		 {
		  if (TasterZeitMessung == 0) TasterZeitMessung = 80; // else 
			 // if (TasterZeitMessung == 1) { TasterStatus = DauerTaster; return (DauerTaster); }
		 } else
		 
		 {
		  if (TasterZeitMessung > 1)
			 {
			  // TasterZeitMessung = 0;
        TasterStatus = KurzTaster;
				return (KurzTaster);
			 }
       if (TasterZeitMessung == 1) { TasterStatus = DauerTaster; return (DauerTaster); }
		 } // Von else if (TasterZustand != 0) 

		return (KeinTaster);
		
 }

//------------------------------------------------------------------------
// Liefert eine positive Flanke vom Bewegungsmelder oder den Türkontakten
//------------------------------------------------------------------------
char GetBewegung (void)
 {
  uint8_t MelderWert = 0;
    MelderKontakt = Port.read8 ();				// Aktuellen Zustand reinsaugen *lutsch*
	
		MelderWert = 0;
		if ( ((MelderKontakt & Bewegungsmelder) != 0) && ((MelderKontaktAlt & Bewegungsmelder) == 0) && (BewegungsmelderEnable) ) MelderWert = 1; // Bewegungsmelder bei Shutdown per Taster unterdrücken
		if ( ((MelderKontakt & ZwischentuerGeoeffnet) != 0) && ((MelderKontaktAlt & ZwischentuerGeoeffnet) == 0) ) 								MelderWert = 2;
		if ( ((MelderKontakt & SchiebetuerGeoeffnet) != 0) && ((MelderKontaktAlt & SchiebetuerGeoeffnet) == 0) ) 								  MelderWert = 3;

		MelderKontaktAlt = MelderKontakt;		 // Alt = Neu für die Flankenerkennung, gelle?

		return (MelderWert);
 }

/***********************************
 *
 * Das gute alte void main (void)
 *
 ***********************************/
void loop () 
 {
 
  //--------------------------
	// Ablaufende Zeit anzeigen
	//--------------------------
  lcd.setCursor (0, 0); 
	if (LichtZaehler > 0)
	 {
		lcd.print (LichtZaehler); 
		lcd.print ("s ");
	 } else lcd.print ("Aus     ");	
  
	//------------------
	// Tasterauswertung
	//------------------
	uint8_t tast = GetTaster ();

  //-------------------------------------
	// Kurz gedrückt? Dann Zeit raufsetzen
	//-------------------------------------
	if (TasterStatus == KurzTaster) 
   {
    TasterStatus = KeinTaster;
    TasterZeitMessung = 0;
		if (LichtZaehler < LichtZaehlerMax) LichtZaehler +=  GrundDauer;
		SollDauerLicht = LichtZaehler;
   }

  //------------------------------------
	// Lang gedrückt? Dann Zeit verkürzen
	//------------------------------------
  if (TasterStatus == DauerTaster) 
   {
    TasterStatus = KeinTaster;
    TasterZeitMessung = 0;
    LichtZaehler =  31;
		BewegungsmelderEnable = 0;
   }
  
 //---------------------------------------------------------------------
 // Türen geöffnet oder Bewegungsmelder angesprochen? Dann nachtriggern
 //---------------------------------------------------------------------
 if (GetBewegung ()) LichtZaehler =  SollDauerLicht;

 //----------------------------
 // Schliesskontakte auswerten
 //----------------------------
 // [...]
 
}
